--[[ Roster module to Handle the Raid Roster    ]]
--[[ tab widths set to 4 for optimal formatting ]]

local _, TriageTable = ...; 

local Triage = TriageTable.Triage
Triage.Roster = Triage:NewModule("Roster", "AceEvent-3.0")
local Roster = Triage.Roster


-- Use active and internal as flip buffers..
-- update one, then swap with the other if there are actual changes.
local active = {
	  raid_roster = { },
	  waiting_roster = { },
};

local internal = {
	  raid_roster = { },
	  waiting_roster = { },
};



function Roster:Debug(str, ...)
	Triage:Debug("|c933f9fffRoster|r - "..str , ... )
end



--[[ Primary module callbacks, initialize, enable, disable ]]
function Roster:OnInitialize()
	-- Called at module load
	self:Debug("OnInitialize")

	self:OnEnable()
end



function Roster:OnEnable()
	-- Called when the module is enabled
	self:Debug("OnEnable")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("UNIT_NAME_UPDATE")

	self:Update()
end



function Roster:OnDisable()
	self:Debug("OnDisable")
	-- Called when the module is disabled
end



--[[ Event handlers ]]
function Roster:PLAYER_ENTERING_WORLD()
	self:Debug("PLAYER_ENTERING_WORLD")
	self:Update()
end



function Roster:RAID_ROSTER_UPDATE()
	self:Debug("RAID_ROSTER_UPDATE")
	self:Update()
end



function Roster:UNIT_NAME_UPDATE()
	self:Debug("UNIT_NAME_UPDATE")
	self:Update()
end



--[[ Update the raid member list ]]
function Roster:Update()
	-- self:Debug("Update")

	if GetNumRaidMembers() == 0 then return end

	local instanceName, _, _, _, maxPlayers = GetInstanceInfo()

	self.instanceName = instanceName
	self:Debug("Instance: "..instanceName)

	-- If we're not actually in the instance, get the max number of players
	-- by looking at the difficulty.. 
	if maxPlayers == 0 then
		difficultyIndex = GetRaidDifficulty()
		self:Debug( ("Difficulty %d"):format(difficultyIndex) )
		if difficultyIndex == 1 or difficultyIndex == 3 then
			maxPlayers = 10
		else
			maxPlayers = 25
		end
	end
	self.instanceSize = maxPlayers
	self:Debug( ("Max Players: %d"):format(maxPlayers) )

	self.maxGroupInRaid = maxPlayers / 5
	self:Debug( ("Max Groups: %d"):format(self.maxGroupInRaid) )

	local updated = false

	table.wipe( internal.raid_roster )
	table.wipe( internal.waiting_roster )
	for i = 1, MAX_RAID_MEMBERS do
		local name, rank, subgroup, level, class, fileName, 
			  zone, online, isDead, role, isML = GetRaidRosterInfo( i )
		local unitid = ("raid%d"):format( i )
		local guid = UnitGUID( unitid )
		if guid then
			if subgroup and subgroup <= self.maxGroupInRaid then
				self:Debug( ("%s in raid (group %d)"):format(name, subgroup) )
				table.insert( internal.raid_roster, name )
			else
				self:Debug( ("%s outside raid (group %d)"):format(name, subgroup) )
				table.insert( internal.waiting_roster, name )
			end
		end
	end
	
	table.sort( internal.raid_roster ) 
	table.sort( internal.waiting_roster ) 
	if #internal.raid_roster ~= #active.raid_roster or
		#internal.waiting_roster ~= #active.waiting_roster then
		updated = true
	else
		for i = 1, #internal.raid_roster do
			if active.raid_roster[i] ~= internal.raid_roster[i] then
				self:Debug(active.raid_roster[i] .. " ~= " .. 
							internal.raid_roster[i] )
				updated = true
				break
			end
		end
	end

	-- If updated, flip the tables.
	if updated then
		local temp = active
		active = internal
		internal = temp
	end

	-- Debugging
	if updated and Triage.debug then
		raid_list = ("Raid size: %d -- Inside:"):format(maxPlayers)
		for i = 1, #active.raid_roster do
    		raid_list = raid_list.." "..active.raid_roster[i]
		end
		raid_list = raid_list.." -- Outside:"
		for i = 1, #active.waiting_roster do
    		raid_list = raid_list.." "..active.waiting_roster[i]
		end
		self:Debug(raid_list)
	end

	if updated then
		self:Debug("Sending RosterUpdated message")
		self.SendMessage("Triage_RosterUpdated")
	end
end



function Roster:GetRaidRoster()
	return active.raid_roster, active.waiting_roster, self.instanceSize
end